from django.conf import settings
from youtuber.exceptions import NoPipelinesError
from youtuber.pipelines import BasePipeline


def get_pipeline_instances():
    try:
        pipeline_classes = settings.YOUTUBE_ENTRY_PIPELINES
    except AttributeError:
        raise NoPipelinesError('No pipelines are configured')

    pipeline_instances = []
    for pipeline_class in pipeline_classes:
        components = pipeline_class.split('.')
        module_name = '.'.join(components[:-1])
        class_name = components[-1]

        module = __import__(module_name, fromlist=[class_name])
        class_ = getattr(module, class_name)
        instance = class_()
        if isinstance(instance, BasePipeline):
            pipeline_instances.append(instance)
    return pipeline_instances



from django.contrib import admin
from youtuber.models import YoutubeSource


class YoutubeSourceAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'type', 'identifier')


admin.site.register(YoutubeSource, YoutubeSourceAdmin)



import requests
from django.conf import settings
from youtuber.exceptions import PlaylistException, ChannelNotFoundException, ClientConfigError


class YoutubeApi(object):
    youtube_api_url = 'https://www.googleapis.com/youtube/v3'

    def __init__(self, developer_key):
        self.developer_key = developer_key

        self.follow_pagination = getattr(settings, 'YOUTUBE_FOLLOW_PAGINATION', False)
        self.max_results = getattr(settings, 'YOUTUBE_MAX_RESULTS', 10)

    def get_username_playlists(self, username):
        parameters = {
            'forUsername': username,
            'part': 'contentDetails'
        }
        uri = 'channels'
        content = self._fetch_content(uri, parameters)

        if 'error' in content:
            raise ClientConfigError(content['error']['message'])

        if content['pageInfo']['totalResults'] == 0:
            raise ChannelNotFoundException()

        if content['items'][0] and content['items'][0]['contentDetails']['relatedPlaylists']:
            return content['items'][0]['contentDetails']['relatedPlaylists']

        return {}

    def get_videos_from_playlist(self, playlist_id, page_token=None, max_results=10):
        parameters = {
            'part': 'snippet',
            'playlistId': playlist_id
        }

        if page_token:
            parameters['pageToken'] = page_token

        if max_results > 0:
            parameters['maxResults'] = max_results
        uri = 'playlistItems'
        response = self._fetch_content(uri, parameters)
        if not response:
            return None

        if 'error' in response:
            raise PlaylistException(response['error']['errors'][0]['message'])

        content = YoutubeFeedResult(response)

        return content

    def _fetch_content(self, uri, parameters):
        parameters['key'] = self.developer_key
        parameters['maxResults'] = self.max_results

        # Retry connections
        for _ in range(3):
            try:
                response = requests.get('%s/%s' % (self.youtube_api_url, uri), parameters)
                return response.json()
            except requests.exceptions.ConnectionError:
                pass
        return None


class YoutubeFeedResult(object):
    def __init__(self, response):
        if 'nextPageToken' in response:
            self.next_page_token = response['nextPageToken']
        else:
            self.next_page_token = None

        if 'prevPageToken' in response:
            self.prev_page_token = response['prevPageToken']
        else:
            self.prev_page_token = None

        if 'pageInfo' in response:
            self.total_results = response['pageInfo']['totalResults']
            self.results_per_page = response['pageInfo']['resultsPerPage']

        self.items = []
        for item in response['items']:
            self.items.append(YoutubeFeedItem(item['snippet']))


class YoutubeFeedItem(object):
    def __init__(self, item):
        self.title = item['title']
        self.description = item['description']
        self.channel_id = item['channelId']
        self.channel_title = item['channelTitle']
        self.playlist_id = item['playlistId']
        self.position = item['position']
        self.published_at = item['publishedAt']
        self.video_id = item['resourceId']['videoId']
        self.images = {}
        if 'thumbnails' in item:
            self.images.update(item['thumbnails'])



from django.db import models
from django.utils.translation import ugettext as _


class YoutubeSource(models.Model):
    TYPE_CHANNEL = 1
    TYPE_PLAYLIST = 2

    TYPES_CHOICES = (
        (TYPE_CHANNEL, _('Channel')),
        (TYPE_PLAYLIST, _('Playlist'))
    )

    name = models.CharField(max_length=250)
    type = models.SmallIntegerField(choices=TYPES_CHOICES)
    identifier = models.CharField(max_length=250)

    date_created = models.DateTimeField(auto_now_add=True)
    date_updated = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return self.name


class ClientConfigError(Exception):
    def __init__(self, message=None):
        super(ClientConfigError, self).__init__(message)


class NoPipelinesError(Exception):
    pass


class ChannelNotFoundException(Exception):
    pass


class PlaylistException(Exception):
    def __init__(self, message):
        super(PlaylistException, self).__init__(message)


from setuptools import setup

setup(
    name='Django-Youtuber',
    version='0.1.1',
    author='Rakan Alhneiti',
    author_email='rakan.alhneiti@gmail.com',

    # Packages
    packages=[
        'youtuber',
        'youtuber/management',
        'youtuber/migrations',
        'youtuber/tests'
    ],
    include_package_data=True,

    # Details
    url='https://github.com/rakanalh/django-youtuber',

    license='LICENSE',
    description='Django app for fetching videos from channels or playlists.',
    long_description=open('README.md').read(),

    # Dependent packages (distributions)
    install_requires=[
        'django',
        'celery',
        'requests'
    ],
)



from django.conf import settings
from celery import shared_task
from celery.utils.log import get_task_logger

from youtuber.client import YoutubeApi
from youtuber.exceptions import ClientConfigError
from youtuber.models import YoutubeSource
from youtuber.utils import get_pipeline_instances


try:
    developer_key = settings.YOUTUBE_DEVELOPER_KEY
except AttributeError:
    raise ClientConfigError('YOUTUBE_DEVELOPER_KEY is not defined')

max_results = getattr(settings, 'YOUTUBE_MAX_RESULTS')
follow_pagination = getattr(settings, 'YOUTUBE_FOLLOW_PAGINATION', False)
max_pagination = getattr(settings, 'YOUTUBE_MAX_PAGINATION', 1)

client = YoutubeApi(developer_key)

@shared_task()
def youtuber_start():
    youtube_sources = YoutubeSource.objects.all()

    logger = get_task_logger(__name__)

    for source in youtube_sources:
        logger.info('Processing youtube source %s' % source.name.encode('utf-8'))

        playlist_id = source.identifier
        if source.type == YoutubeSource.TYPE_CHANNEL:
            playlists = client.get_username_playlists(source.identifier)

            if 'uploads' in playlists:
                playlist_id = playlists['uploads']
            else:
                logger.info('No uploads channel found')
                return

        youtuber_scrape.delay(source_id=source.id, playlist_id=playlist_id)


@shared_task()
def youtuber_scrape(source_id, playlist_id, page_token=None):
    logger = get_task_logger(__name__)

    pipeline_instances = get_pipeline_instances()
    logger.info('Got %d pipelines' % len(pipeline_instances))

    youtube_source = YoutubeSource.objects.get(pk=source_id)

    for _ in range(max_pagination):
        videos = client.get_videos_from_playlist(playlist_id, page_token, max_results)

        if not videos:
            break

        logger.info('Extracted %d videos' % len(videos.items))

        for video in videos.items:
            for instance in pipeline_instances:
                logger.info('Calling %s pipeline for video %s' % (instance.__class__, video.title))
                instance.process(youtube_source, video)

        if not follow_pagination or not videos.next_page_token:
            break

        page_token = videos.next_page_token

    return
